/*
 * JanelaTabela.java
 *
 * Created on 29 de Setembro de 2006, 17:59
 */

package parte03;

import classes.Camisa;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

/**
 *
 * @author  vitor
 */
public class JanelaTabelaModelo extends javax.swing.JFrame {
    ModeloTabelaProdutos modelo;
    
    /** Creates new form JanelaTabela */
    public JanelaTabelaModelo() {
        initComponents();
        initModel();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        scrollPane = new javax.swing.JScrollPane();
        table = new javax.swing.JTable();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Tabela");
        table.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        scrollPane.setViewportView(table);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(scrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 376, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(scrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 276, Short.MAX_VALUE)
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    private void initModel() {
        // Obtém os produtos a serem exibidos.
        Collection<Camisa> produtos = Camisa.listProdutos();
        
        // Cria um modelo personalizado a partir dos produtos.
        modelo = new ModeloTabelaProdutos(produtos);
        table.setModel(modelo);
        
        // Adiciona o renderizador padrão para cores.
        table.setDefaultRenderer(Color.class, new ColorTableCellRenderer());
        
        // Configura a altura da linha.
        table.setRowHeight(60);
        
        // Adiciona um ouvinte de mouse à tabela.
        table.getTableHeader().addMouseListener(new SortListener());
        
        // Substitui o editor padrão do tamanho da camisa por uma combo box.
        JComboBox comboTamanho = new JComboBox(new Character[] { 'P', 'M', 'G' });
        TableCellEditor editorTamanho = new DefaultCellEditor(comboTamanho);
        TableColumnModel modeloColunas = table.getColumnModel();
        TableColumn colunaTamanho = modeloColunas.getColumn(1);
        colunaTamanho.setCellEditor(editorTamanho);
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new JanelaTabelaModelo().setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane scrollPane;
    private javax.swing.JTable table;
    // End of variables declaration//GEN-END:variables
 
    private class SortListener extends MouseAdapter {
        public void mouseClicked(MouseEvent event) {
            // Localiza a coluna do clique.
            int coluna = table.getTableHeader().columnAtPoint(event.getPoint());

            // Converte o índice da tabela para o índice do modelo.
            int indiceModelo = table.convertColumnIndexToModel(coluna);

            // Ordena o modelo pela coluna.
            modelo.ordena(indiceModelo);
        }
    }
}

class ModeloTabelaProdutos extends AbstractTableModel {
    private List<Camisa> produtos;
    private static NumberFormat precoFormat = NumberFormat.getCurrencyInstance();
    
    public ModeloTabelaProdutos(Collection<Camisa> produtos) {
        this.produtos = new ArrayList<Camisa>(produtos);
    }
    
    public int getRowCount() {
        return produtos.size();
    }

    public int getColumnCount() {
        return 6;
    }
    
    public String getColumnName(int columnIndex) {
        switch (columnIndex) {
            case 0: return "Modelo";
            case 1: return "Tamanho";
            case 2: return "Cor";
            case 3: return "Preço";
            case 4: return "Foto";
            case 5: return "Baby Look";
        }
        return super.getColumnName(columnIndex);
    }

    public Class getColumnClass(int columnIndex) {
        switch (columnIndex) {
            case 0: return String.class;
            case 1: return Character.class;
            case 2: return Color.class;
            case 3: return String.class;
            case 4: return ImageIcon.class;
            case 5: return Boolean.class;
        }
        return super.getColumnClass(columnIndex);
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Camisa c = produtos.get(rowIndex);
        switch (columnIndex) {
            case 0: return c.getModelo();
            case 1: return c.getTamanho();
            case 2: return c.getCor();
            case 3: return precoFormat.format(c.getPreco());
            case 4: return c.getFoto();
            case 5: return c.getBabyLook();
        }
        return null;
    }
    
    public void ordena(int columnIndex) {
        Comparator<Camisa> comparador = null;
        switch (columnIndex) {
            case 0: comparador = new Camisa.ComparadorModelo(); break;
            case 1: comparador = new Camisa.ComparadorTamanho(); break;
            case 3: comparador = new Camisa.ComparadorPreco(); break;
       }
       if (comparador != null) {
            Collections.sort(produtos, comparador);
            fireTableDataChanged();
       }
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return (columnIndex == 1) || (columnIndex == 3);
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        Camisa c = produtos.get(rowIndex);
        
        // Preço e tamanho são editáveis.
        switch (columnIndex) {
            case 1:
                c.setTamanho(aValue.toString().charAt(0));
                break;
                
            case 3: 
                try {
                    Number n = precoFormat.parse(aValue.toString());
                    c.setPreco(n.doubleValue());
                }
                catch (ParseException e) {
                    JOptionPane.showMessageDialog(null, "Valor inválido para preço. Tente novamente por favor.");
                }
                break;
       }
    }
}

class ColorTableCellRenderer implements TableCellRenderer {
    private JPanel panel = new JPanel();
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            panel.setBackground((Color)value);
            return panel;
    }
}